/*******************************************************************************
*
*	TITLE:		led_pwm.c 
*
*	VERSION:	1.1                          
*
*	DATE:		24-Mar-2004
*
*	AUTHOR:		R. Kevin Watson
*
*	COMMENTS:
*
********************************************************************************
*
*	Change log:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	23-Mar-2004  1.0  RKW Original
*	24-Mar-2004  1.1  RKW - Fixed bug where the PR2 register was loaded with
*	                  the incorrect value.
*
*******************************************************************************/

#include "ifi_picdefs.h"
#include "led_pwm.h"

/*******************************************************************************
*
*	FUNCTION:		Initialize_LED_PWM()
*
*	PURPOSE:		Initializes the LED PWM code.
*
*	CALLED FROM:	user_routines.c/User_Initialization()
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:		This function initializes the PIC18F8520 CCP2 & CCP3 modules
*					to generate a free running 40KHz pulse train on the EDU-RC's 
*					PWM1 & PWM2	output pins. Once running, the duty cycle of the
*					PWM outputs can the altered on-the-fly by updating the CCPR2L
*					and CCPR3L registers. The higher the value placed in the
*					CCPRxL register, the more time the LED will remain on, making
*					it appear brighter.
*
*					To use this code:
*
*					1) Place a call to Initialize_LED_PWM() in user_routines.c/
*					User_Initialization().
*
*					2) Change the first two parameters in Setup_Who_Controls_Pwms()
*					from 'Master' to 'User'.
*
*					3) Change the first two parameters in Setup_PWM_Output_Type()
*					from 'IFI_PWM' to 'USER_CCP'.
*
*					4) Add '#include "led_pwm.h"' to the top of user_routines.c.
*
*******************************************************************************/
void Initialize_LED_PWM(void)  
{
	// initialize PWM output 1
	TRISCbits.TRISC1 = 0;	// make sure CCP2/PWM1 is setup as an output [110]
	CCPR2L = 0;				// set initial CCP2/PWM 1 duty cycle [154]
	CCP2CON = 0x0C;			// put CCP module 2 into PWM mode [149]

	// initialize PWM output 2
	TRISGbits.TRISG0 = 0;	// make sure CCP3/PWM2 is setup as an output [121]
	CCPR3L = 0;				// set initial CCP3/PWM 2 duty cycle [154]
	CCP3CON = 0x0C;			// put CCP module 3 into PWM mode [149]
		
	// initialize timer 2
	TMR2 = 0;				// 8-bit timer 2 register (this is readable and writable)
							//
	PR2	= 249;				// timer 2 period register - timer 2 increments to this 
							// value then resets to zero on the next clock and starts
							// all over again. Set to (10000000/desired frequency) - 1.
							//
	T2CONbits.T2OUTPS0 = 0;	// T2OUTPS3 T2OUTPS2 T2OUTPS1 T2OUTPS0
	T2CONbits.T2OUTPS1 = 0;	//    0        0        0        0		1:1 postscaler
	T2CONbits.T2OUTPS2 = 0;	//    0        0        0        1		1:2 postscaler
	T2CONbits.T2OUTPS3 = 0;	//    0        0        1        0		1:3 postscaler
							//    0        0        1        1		1:4 postscaler
							//    0        1        0        0		1:5 postscaler
							//    0        1        0        1		1:6 postscaler
							//    0        1        1        0		1:7 postscaler
							//    0        1        1        1		1:8 postscaler
							//    1        0        0        0		1:9 postscaler
							//    1        0        0        1		1:10 postscaler
							//    1        0        1        0		1:11 postscaler
							//    1        0        1        1		1:12 postscaler
							//    1        1        0        0		1:13 postscaler
							//    1        1        0        1		1:14 postscaler
							//    1        1        1        0		1:15 postscaler
							//    1        1        1        1		1:16 postscaler
							//
	T2CONbits.T2CKPS0 = 0;	// T2CKPS1  T2CKPS0
	T2CONbits.T2CKPS1 = 0;	//    0        0	1:1 prescaler (clock = 10MHz/each tick=100ns)
							//    0        1	1:4 prescaler (clock = 2.5MHz/each tick=400ns)
							//    1        x	1:16 prescaler (clock = 625KHz/each tick=1.6us) (T2CKPS0 doesn't matter)
							//
	IPR1bits.TMR2IP = 0;	// 0: timer 2 interrupt is low priority (leave at 0 for IFI controllers)
							// 1: timer 2 interrupt is high priority
							//
	PIR1bits.TMR2IF = 0;	// 0: TMR2 to PR2 match hasn't happened (set to 0 before enabling the interrupt)
							// 1: TMR2 to PR2 match has happened
							//
	PIE1bits.TMR2IE = 0;	// 0: disable timer 2 interrupt on PR2 match
							// 1: enable timer 2 interrupt on PR2 match
							//    if the prescaler is enabled (i.e., greater than 1:1), this
							//    match will occur n times (where n is the postscaler value)
							//    before an interrupt will be generated
							//
	T2CONbits.TMR2ON = 1;	// 0: timer 2 is disabled
							// 1: timer 2 is enabled (running)
}
